@@ -118,16 +118,18 @@
  * [1]: https://arxiv.org/abs/2201.02179
  */
 
-#include "prelude.h"
 #include "ioq.h"
+
 #include "alloc.h"
 #include "atomic.h"
+#include "bfs.h"
 #include "bfstd.h"
 #include "bit.h"
 #include "diag.h"
 #include "dir.h"
 #include "stat.h"
 #include "thread.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <pthread.h>
@@ -180,8 +182,7 @@ typedef atomic uintptr_t ioq_slot;
 /** Amount to add for an additional skip. */
 #define IOQ_SKIP_ONE (~IOQ_BLOCKED)
 
-// Need room for two flag bits
-bfs_static_assert(alignof(struct ioq_ent) >= (1 << 2));
+static_assert(alignof(struct ioq_ent) >= (1 << 2), "struct ioq_ent is underaligned");
 
 /**
  * An MPMC queue of I/O commands.
